;;; ydg.scm --- Yellow Draw Graphic file format

;;;;   Copyright (C) 1998 GYVE Development Team
;;;;
;;;;   Author: Masatake YAMATO (masata-y@is.aist-nara.ac.jp)
;;;;   Created: Wed Oct 29 01:13:55 1997
;;;;   Time-stamp: <98/06/12 02:54:01 masata-y>
;;;;   
;;;;   This file is part of the GYVE vector based drawing editor. 
;;;;
;;;;   This program conforms GNU GENERAL PUBLIC LICENSE Version 2.

(define-module (gyve ydg)
  :use-module (gyve figs)
  :use-module (languages guileobjc))

(define-public ydg:document #f)

(define shared:filename #f)
(define shared:pagetable #f)
(define shared:pagenumber 0)
(define shared:application ([] (objc-class Gyve) application))

(define (shared:custom-color-dict)
  ([] shared:application customColorDictionary))
(define (shared:pattern-dict)
  ([] shared:application patternDictionary))
(define (shared:gradation-dict)
  ([] shared:application gradationDictionary))

(define shared:gopi0 (objc-new GOPIPoint init))
(define (shared:gopi x y)
  ([] shared:gopi0 setX: x)
  ([] shared:gopi0 setY: y)
  shared:gopi0)

(define-public (ydg:set-version str)
  #f)
(define-public (ydg:newpage h w . layers)
  (let* ((buffer (objc-new GyveBuffer 
			   initWithCStringName: 
			   (string-append shared:filename 
					  (number->string 
					   shared:pagenumber))
			   andHeight: h andWidth: w
			   )))
    (set! shared:pagenumber (+ 1 shared:pagenumber))
    ([] shared:pagetable addObject: buffer)
    (map (lambda (alayer)
	   ([] buffer putLayerOnTop: alayer))
	 layers)
    buffer
    ))

(define-public (ydg:newlayer . figs)
  (let ((layer (objc-new GyveLayer init)))
    (map (lambda (afig)
	   ([] layer putFigObjOnTop: afig))
	 figs)
    layer))

(define-public (ydg:newgroup . figs)
  (let ((group (objc-new GyveFigGroup init)))
    (map (lambda (afig)
	   ([] group putFigObjOnTop: afig))
	 figs)
    group))

(define-public (ydg:newmask mask . figs)
  (let ((maskobj (objc-new GyveMask initWithMaskPath: mask)))
    (map (lambda (afig)
	   ([] maskobj putFigObjOnTop: afig))
	 figs)
    maskobj))

(define-public (ydg:compound-paths base . figs)
  (let ((compound (objc-new GyveCompoundPaths initWithFigObj: base)))
    (map (lambda (afig)
	   ([] compound putFigObjOnTop: figs))
	 figs)
    compound))

;; XXX render mode
(define-public (ydg:path paintstyle rendermode . segments)
  (let ((path (objc-new PSPath initWithStartSegment: (car segments))))
    (map (lambda (seg)
	   ([] path addSegment: seg))
	 (cdr segments))
    ([] path setPaintStyle: paintstyle)
    (if (< 0 rendermode)		; bit operator is needed
	([] path openPath: #f))
    path))

(define-public (ydg:lineto x y)
  ([] (objc-new PSLinetoSegment init) setCoordsX: x andY: y))
(define-public (ydg:moveto x y)
  ([] (objc-new PSMovetoSegment init) setCoordsX: x andY: y))
(define-public (ydg:curveto x1 y1 x2 y2 x3 y3)
  ([] (objc-new PSCurvetoSegment init) setCoordsX1: x1 andY1: y1
      X2: x2 andY2: y2
      X3: x3 andY3: y3))
(define-public (ydg:path-attribute dash	;obj
				   flatness	; flat
				   linejoin	; int
				   linecap	; int
				   miterlimit	; float
				   linewidth)	; float
  (let ((pattr (objc-new PSPathAttributes init)))
    ([] pattr setDashPattern: dash)
    ([] pattr setFlatness: flatness)
    ([] pattr setLineJoin: linejoin)
    ([] pattr setLineCap: linecap)
    ([] pattr setMiterLimit: miterlimit)
    ([] pattr setLineWidth: linewidth)
    pattr))

(define-public (ydg:dash-pattern phase . pattern)
  (let ((len (length pattern))
	(dash #f)
	(i 0))
    (if (and (eq? phase 0)
	     (eq? len 0))
	(objc-get-nil)
	(begin 
	  (set! dash (objc-new PSDynamicDashPattern
			       initWithDashArrayLength: 
			       len))
	  ([] dash setDashPhase: phase)
	  (for-each (lambda (x)
		     ([] dash setDashArray: x atIndex: i)
		     (set! i (+ 1 i)))
		    pattern)
	  dash))))

(define-public (ydg:paint-style attr		; obj
			fillapp		; obj
			strokeapp)	; obj
  (let ((ptype (objc-new PSPaintStyle initWithPathAttributes: attr)))
    (if (not (eq? fillapp #t))
	([] ptype setFillingPaintingApplication: fillapp))
    (if (not (eq? strokeapp #t))
	([] ptype setStrokingPaintingApplication: strokeapp))
    ptype))

(define-public (ydg:gradation-app name ftype begx begy endx endy crossx crossy)
  (let ((gapp (objc-new 
	       PSGradationApplication initWithKey: (objc-nsstring name)
	       andGradationDictionary: (shared:gradation-dict))))
    ([] gapp setBeginningPointByGOPI: (shared:gopi begx begy))
    ([] gapp setEndPointByGOPI: (shared:gopi endx endy))
    ([] gapp setCrossPointByGOPI: (shared:gopi crossx crossy))
    ([] gapp setFillingType: ftype)
    gapp))

(define-public (ydg:pattern-app name a b c d tx ty)
  (let* ((papp (objc-new PSPatternApplication 
			 initWithKey: (objc-nsstring name)
			 andPatternDictionary: (shared:pattern-dict)))
	 (matrix ([] papp identityMatrix)))
    ([] matrix setValue: a atIndex: 0)
    ([] matrix setValue: b atIndex: 1)
    ([] matrix setValue: c atIndex: 2)
    ([] matrix setValue: d atIndex: 3)
    ([] matrix setValue: tx atIndex: 4)
    ([] matrix setValue: ty atIndex: 5)
    papp))

(define-public (ydg:custom-color name)
  (objc-new PSCustomColor initWithKey: (objc-nsstring name)
	    andColorDictionary: (shared:custom-color-dict)))

(define-public (ydg:process-color r g b)
  (objc-new PSProcessColor initWithRed: r green: g blue: b))

(define-public (ydg:miter-join) 0)
(define-public (ydg:round-join) 1)
(define-public (ydg:beveled-join) 2)

(define-public (ydg:butt-end-caps) 0)
(define-public (ydg:round-end-caps) 1)
(define-public (ydg:square-end-caps) 2)

(define-public (gyve:ydg:read filename pagetable)
  (set! shared:filename filename)
  (set! shared:pagenumber 0)
  (set! shared:pagetable pagetable)
  (call-with-input-file filename
    (lambda (port)
      (do ((exp (read port) (read port)))
	  ((eof-object? exp))
	(ydg:eval exp)))))

(define (ydg:eval exp)
  (cond ((or (boolean? exp)
	     (char? exp)
	     (vector? exp)
	     (number? exp)
	     (string? exp))
	 exp)

	;; for compatibility
	((and (pair? exp)
	      (eq? (car exp) 'use-modules))
	 'ignore)
	((and (pair? exp)
	      (eq? (car exp) 'let))
	 ;; ignoring variable bindings (cadr exp)
	 (let ((result 'unknown))
	   (for-each (lambda (x) (set! result (ydg:eval x)))
		     (cddr exp))
	   result))

	((pair? exp)
	 (or (list? exp)
	     (error "bad form" exp))
	 (let ((ls (map ydg:eval exp)))
	   (or (procedure? (car ls))
	       (error "bad form" exp))
	   (apply (car ls) (cdr ls))))
	((symbol? exp)
	 (case exp
	   ((defcustom) gyve:figs:define-custom-color)
	   ((defgraddation) gyve:figs:define-gradation)
	   ((defpattern) gyve:figs:define-pattern)

	   ((page) ydg:newpage)
	   ((layer) ydg:newlayer)
	   ((group) ydg:newgroup)
	   ((mask) ydg:newmask)
	   ((compound) ydg:compound-paths)
	   ((path) ydg:path)

	   ((paintstyle) ydg:paint-style)

	   ((gradapp) ydg:gradation-app)
	   ((patapp) ydg:pattern-app)
	   ((custom) ydg:custom-color)
	   ((process) ydg:process-color)

	   ((moveto) ydg:moveto)
	   ((curveto) ydg:curveto)
	   ((lineto) ydg:lineto)

	   ((pathattr)  ydg:path-attribute)
	   ((dash)   ydg:dash-pattern)

	   ((mj) ydg:miter-join)
	   ((rj) ydg:round-join)
	   ((bj) ydg:beveled-join)

	   ((bec) ydg:butt-end-caps)
	   ((rec) ydg:round-end-caps)
	   ((sec) ydg:square-end-caps)
	   ((version) ydg:set-version)
	   ((bar) 0)
	   ((circle) -1)
	   ((nil) (objc-get-nil))
	   (else
	    (error "invalid symbol" exp))))
	(else
	 (error "unknown type" exp))))
